/*
 * gbiff2 program
 * Copyright (C) 2000-2002 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#include "gbiff2.h"
#include "interface.h"
#include "callbacks.h"
#include "support.h"
#include "interface.h"


/* Mailbox toplevel */
/********************************************************************************
 * Function: Callback on a generic change in the interface
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Modify the interface according to biff
 * Return  : /
 ********************************************************************************/
void on_generic_change (GtkWidget *widget, gpointer user_data) {
  if (((Biff *) user_data)->block)
    return;
  updateBiffFromInterface (user_data);
  updateBiffToInterface (user_data);
}

/********************************************************************************
 * Function: Callback on configuration dialog deletion
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Show the applet in gnome taskbar or the biff window
 *           Remove the poll timer and initiate a new one
 *           Hide the mailbox 
 * Return  : TRUE
 ********************************************************************************/
gboolean on_ConfigurationDialog_delete_event (GtkObject *object, GdkEvent *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  updateBiffFromInterface (biff);
  updateBiffToInterface (biff);
  gtk_widget_show (biff->biff);
# ifndef USE_GNOME
  my_gtk_window_parse_geometry (GTK_WINDOW(biff->biff), biff->biffGeometry);
# endif

  if (biff->pollMode == AUTOMATIC_MODE) {
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      if (biff->debug)
	g_message ("on_ConfigurationDialog_delete_event: removing poll timer\n");
    }
    biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
    if (biff->debug)
      g_message ("on_ConfigurationDialog_delete_event: adding poll timer (%d)\n", biff->polltime);
  }
  gtk_widget_hide (WIDGET ("ConfigurationDialog"));
  saveConfig (biff);

#ifndef USE_GNOME
  updateBiff (biff);
#else
  updateApplet (biff);
#endif

  return (TRUE);
}

/********************************************************************************
 * Function: Callback on address button clicked
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->address
 * Return  : /
 ********************************************************************************/
void on_AddressButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  biff->action = 0;
  gtk_widget_show (WIDGET ("FileSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->address);
    biff->address =
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (WIDGET("FileSelectionDialog"))));
    updateBiffToInterface (user_data);
  }
}

/********************************************************************************
 * Function: Callback on mail application button selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->mailApp
 * Return  : /
 ********************************************************************************/
void on_MailAppButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FileSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->mailApp);
    biff->mailApp =
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (WIDGET("FileSelectionDialog"))));
    updateBiffToInterface (biff);
  }
}

/********************************************************************************
 * Function: Callback on sound button selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->soundFile
 * Return  : /
 ********************************************************************************/
void on_SoundButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FileSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->soundFile);
    biff->soundFile =
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (WIDGET("FileSelectionDialog"))));
    updateBiffToInterface (biff);
  }
}

/********************************************************************************
 * Function: Callback on sound play button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Play the current sound using system command "play"
 * Return  : /
 ********************************************************************************/
void on_SoundPlayButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  char command[1048];
  
  /* Do not remove the 1024, it prevents buffer overflow */
  sprintf (command, "play %.4080s -v %f",
	   biff->soundFile,
	   gtk_range_get_value (GTK_RANGE (WIDGET ("VolumeHScale")))/100.0);
  system (command);
}

/********************************************************************************
 * Function: Callback on popup title font button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->titleFont
 * Return  : /
 ********************************************************************************/
void on_PopupTitleButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FontSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->titleFont);
    biff->titleFont = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (WIDGET("FontSelectionDialog")));
    updateBiffToInterface (biff);
  }
}

/********************************************************************************
 * Function: Callback on popup body font button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->bodyFont
 * Return  : /
 ********************************************************************************/
void on_PopupBodyButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FontSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->bodyFont);
    biff->bodyFont = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (WIDGET("FontSelectionDialog")));
    updateBiffToInterface (biff);
  }
}

/********************************************************************************
 * Function: Callback on biff font button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->biffFont
 * Return  : /
 ********************************************************************************/
void on_BiffButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show(WIDGET ("FontSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->biffFont);
    biff->biffFont = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (WIDGET("FontSelectionDialog")));
    updateBiffToInterface (biff);
  }
}

/********************************************************************************
 * Function: Callback on mail image selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->mailImageFile
 * Return  : FALSE
 ********************************************************************************/
gboolean on_MailEventBox_button_press_event (GtkWidget *w, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FileSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->mailImageFile);
    biff->mailImageFile = 
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (WIDGET("FileSelectionDialog"))));
    updateBiffToInterface (biff);
  }
  return FALSE;
}

/********************************************************************************
 * Function: Callback on nomail image selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->nomailImageFile
 * Return  : FALSE
 ********************************************************************************/
gboolean on_NoMailEventBox_button_press_event (GtkWidget *wt, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  biff->action = 0;
  gtk_widget_show (WIDGET ("FileSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->nomailImageFile);
    biff->nomailImageFile = 
      g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (WIDGET("FileSelectionDialog"))));
    updateBiffToInterface (biff);
  }	
  return FALSE;
}

/********************************************************************************
 * Function: Callback on read mail color selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->readColor
 * Return  : FALSE
 ********************************************************************************/
gboolean on_ReadEventBox_button_press_event (GtkWidget *w, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  GdkColor color;
  gchar buffer [16];

  biff->action = 0;
  gtk_widget_show (WIDGET ("ColorSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->readColor);
    gtk_color_selection_get_current_color 
      (GTK_COLOR_SELECTION ((GTK_COLOR_SELECTION_DIALOG (WIDGET("ColorSelectionDialog")))->colorsel), &color);
    sprintf (buffer, "#%.2x%.2x%.2x", color.red/256, color.green/256, color.blue/256);
    biff->readColor = g_strdup (buffer);
    updateBiffToInterface (biff);
  }	
  return FALSE;
}

/********************************************************************************
 * Function: Callback on nomail image selection
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : biff->unreadeColor
 * Return  : FALSE
 ********************************************************************************/
gboolean on_UnreadEventBox_button_press_event (GtkWidget *wt, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  GdkColor color;
  gchar buffer [16];

  biff->action = 0;
  gtk_widget_show (WIDGET ("ColorSelectionDialog"));
  gtk_main();
  if (biff->action == 1) {
    g_free (biff->unreadColor);
    gtk_color_selection_get_current_color 
      (GTK_COLOR_SELECTION ((GTK_COLOR_SELECTION_DIALOG (WIDGET("ColorSelectionDialog")))->colorsel), &color);
    sprintf (buffer, "#%.2x%.2x%.2x", color.red/256, color.green/256, color.blue/256);
    biff->unreadColor = g_strdup (buffer);
    updateBiffToInterface (biff);
  }	
  return FALSE;
}

/********************************************************************************
 * Function: Callback on OK button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Shows the applet (GNOME) or window (GTK)
 *           Lauch poll timer and hide mailbox properties
 * Return  : /
 ********************************************************************************/
void on_OkButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  updateBiffFromInterface (biff);
  updateBiffToInterface (biff);

  gtk_widget_show (biff->biff);
# ifndef USE_GNOME
  my_gtk_window_parse_geometry (GTK_WINDOW(biff->biff), biff->biffGeometry);
# endif

  if (biff->pollMode == AUTOMATIC_MODE) {
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      if (biff->debug)
	printf ("on_OkButton_clicked: removing poll timer\n");
    }
    biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
    if (biff->debug)
      printf ("on_OkButton_clicked: adding poll timer (%d)\n", biff->polltime);
  }
  gtk_widget_hide (WIDGET ("ConfigurationDialog"));
  saveConfig (biff);

#ifndef USE_GNOME
  updateBiff (biff);
#else
  updateApplet (biff);
#endif
}

/********************************************************************************
 * Function: Callback on Quit Button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Free gbiff stuff and quit
 * Return  : /
 ********************************************************************************/
void on_QuitButton_clicked (GtkButton *button, gpointer user_data) {
  gtk_main_quit ();
  exit (0);
}

/********************************************************************************
 * Function: Callback on Suspend Button
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Suspend automatic mail check
 * Return  : /
 ********************************************************************************/
void on_SuspendButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  
  if (biff->pollMode == AUTOMATIC_MODE) {
    gtk_label_set_text (GTK_LABEL (WIDGET ("SuspendButtonLabel")), "Continue");
    gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("SuspendButtonImage")),
			       gdk_pixbuf_new_from_file (GBIFF2_DATADIR"/timer.xpm", 0));
    biff->pollMode = MANUAL_MODE;
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      biff->polltag = 0;
      if (biff->debug)
	printf ("on_SuspendButton_clicked: removing poll timer\n");
    };
  }
  else {
    gtk_label_set_text (GTK_LABEL (WIDGET ("SuspendButtonLabel")), "Suspend");
    gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("SuspendButtonImage")),
			       gdk_pixbuf_new_from_file (GBIFF2_DATADIR"/timer_stopped.xpm", 0));
    biff->pollMode = AUTOMATIC_MODE;
    if (biff->polltag == 0) {
      biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
      if (biff->debug)
	printf ("on_SuspendButton_clicked: adding poll timer (%d)\n", biff->polltime);
    }
  }
}





/* Popup */
/********************************************************************************
 * Function: Callback on popup button press event
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Hide popup, remore pop timer, set poll timer
 * Return  : TRUE
 ********************************************************************************/
gboolean on_Popup_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
  if (event->button == 3) {
    popdown (user_data);
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/********************************************************************************
 * Function: Callback on popup enter event
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Remove pop timer
 * Return  : TRUE
 ********************************************************************************/
gboolean on_Popup_enter_notify_event (GtkWidget *widget, GdkEventCrossing *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  
  if (biff->poptag > 0) {
    gtk_timeout_remove (biff->poptag);
    biff->poptag = 0;
  }
  return TRUE;
}

/********************************************************************************
 * Function: Callback on popup leave event
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Set pop timer
 * Return  : TRUE
 ********************************************************************************/
gboolean on_Popup_leave_notify_event (GtkWidget *widget, GdkEventCrossing *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  GdkRectangle rect;
  gint result;
  gint x, y;
  GString *geometry = g_string_new("");

  if (GTK_WIDGET_VISIBLE (biff->popup)) {
    if (biff->poptag > 0)
      gtk_timeout_remove (biff->poptag);
    biff->poptag = gtk_timeout_add (biff->poptime*1000, on_TimeoutPop_event, biff);

    gdk_window_get_frame_extents (biff->popup->window, &rect);
    if (biff->debug) {
      g_message ("current geometry: %s", biff->popupGeometry);
      g_message ("window position : %dx%d", rect.x, rect.y);
      g_message ("window size     : %dx%d", rect.width, rect.height);
    }

    result = gtk_XParseGeometry (biff->popupGeometry, &x, &y, 0, 0);
    if ((result & XNegative) && (result & YNegative))
      g_string_printf (geometry, "-%d-%d",
		       MAX(gdk_screen_width()-rect.width-rect.x,0),
		       MAX(gdk_screen_height()-rect.height-rect.y,0));
    else if (result & XNegative)
      g_string_printf (geometry, "-%d+%d",
		       MAX(gdk_screen_width()-rect.width-rect.x, 0),
		       MIN(rect.y, gdk_screen_height()-rect.height));
    else if (result & YNegative)
      g_string_printf (geometry, "+%d-%d",
		       MIN(rect.x, gdk_screen_width()-rect.width),
		       MAX(gdk_screen_height()-rect.height-rect.y, 0));
    else
      g_string_printf (geometry, "+%d+%d",
		       MIN(rect.x, gdk_screen_width()-rect.width),
		       MIN(rect.y, gdk_screen_height()-rect.height));
    if (biff->debug)
      g_message ("guessed geometry: %s", geometry->str);
    g_free (biff->popupGeometry);
    biff->popupGeometry = g_strdup (geometry->str);
    g_string_free (geometry, TRUE);
    updateBiffToInterface (biff);
  };

  return TRUE;
}


/* Biff */
/********************************************************************************
 * Function: Get current biff position and update configuration dialog
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Configuration dialog
 * Return  : TRUE
 ********************************************************************************/
gboolean on_BiffDialog_leave_notify_event (GtkWidget *widget, GdkEventCrossing *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  GdkRectangle rect;
  gint result;
  gint x, y;
  GString *geometry = g_string_new("");

  gdk_window_get_frame_extents (biff->biff->window, &rect);
  if (biff->debug) {
    g_message ("current geometry: %s", biff->biffGeometry);
    g_message ("window position : %dx%d", rect.x, rect.y);
    g_message ("window size     : %dx%d", rect.width, rect.height);
  }

  result = gtk_XParseGeometry (biff->biffGeometry, &x, &y, 0, 0);
  if ((result & XNegative) && (result & YNegative))
    g_string_printf (geometry, "-%d-%d",
		     MAX(gdk_screen_width()-rect.width-rect.x,0),
		     MAX(gdk_screen_height()-rect.height-rect.y,0));

  else if (result & XNegative)
    g_string_printf (geometry, "-%d+%d",
		     MAX(gdk_screen_width()-rect.width-rect.x, 0),
		     MIN(rect.y, gdk_screen_height()-rect.height));
  else if (result & YNegative)
    g_string_printf (geometry, "+%d-%d",
		     MIN(rect.x, gdk_screen_width()-rect.width),
		     MAX(gdk_screen_height()-rect.height-rect.y, 0));
  else
    g_string_printf (geometry, "+%d+%d",
		     MIN(rect.x, gdk_screen_width()-rect.width),
		     MIN(rect.y, gdk_screen_height()-rect.height));


  if (biff->debug)
    g_message ("guessed geometry: %s", geometry->str);

  g_free (biff->biffGeometry);
  biff->biffGeometry = g_strdup (geometry->str);
  g_string_free (geometry, TRUE);

  updateBiffToInterface (biff);
  return TRUE;
}

/********************************************************************************
 * Function: Callback on window expose event
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Update window
 * Return  : FALSE
 ********************************************************************************/
gboolean on_BiffDA_expose_event (GtkWidget *w, GdkEventExpose *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  // Draw only last expose
  if (event->count > 0)
    return (TRUE);

  updateBiff (biff);
  return FALSE;
}

/********************************************************************************
 * Function: Callback on window configure event
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Update window
 * Return  : TRUE
 ********************************************************************************/
gboolean on_BiffDA_configure_event (GtkWidget *w, GdkEventConfigure *event, gpointer user_data) {
  return TRUE;
}

/********************************************************************************
 * Function: Callback on window button press
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Button 1 => get mail status and show mail headers if necessary
 *           Button 3 => show mailbox properties
 * Return  : FALSE
 ********************************************************************************/
gboolean on_BiffDA_button_press_event (GtkWidget *w, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  char command[1048];

  /* Single click on button 3 */
  /* Raise the configuration dialog */
  if (event->button == 3) {
    gtk_widget_hide (biff->popup);
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      biff->polltag = 0;
      if (biff->debug)
	printf ("on_BiffDA_button_press: removing poll timer\n");
    }
    if (biff->poptag > 0) {
      gtk_timeout_remove (biff->poptag);
      biff->poptag = 0;
    }
    gtk_widget_show (WIDGET("ConfigurationDialog"));
  }


  /* Single click on button 2 */
  /* Launch the mail application */
  if (event->button == 2) {
    /* Do not remove the 1024, it prevents buffer overflow */
    sprintf (command, "%.1024s &", biff->mailApp);
    system (command);
  }


  /* Single click on button 1 */
  /* Force mail check */
  if (event->button == 1) {
    if (biff->poptag > 0) {
      gtk_timeout_remove (biff->poptag);
      biff->poptag = 0;
    }
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      biff->polltag = 0;
      if (biff->debug)
	printf ("on_BiffDA_button_press: removing poll timer\n");
    }


    /* If the popup was on display, then the event is a request to hide it */
    if (GTK_WIDGET_VISIBLE(biff->popup))
      popdown (biff);
    else
      checkMail (biff, 1);
  }	
  
  return TRUE;
}

/********************************************************************************
 * Function: Callback on window button release
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Set pop timer if popup is on
 * Return  : FALSE
 ********************************************************************************/
gboolean on_BiffDA_button_release_event (GtkWidget *widget, GdkEventButton *event,  gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  if (GTK_WIDGET_VISIBLE (biff->popup)) {
    if (biff->poptag > 0)
      gtk_timeout_remove (biff->poptag);
    biff->poptag = gtk_timeout_add (biff->poptime*1000, on_TimeoutPop_event, biff);
  }

  return FALSE;
}


/* Timers */
/********************************************************************************
 * Function: Callback on poll timeout
 * Input   : user_data which is a biff structure
 * Output  : Check mail status and get headers if necessary
 *           Show the popup and set timers stuff
 * Return  : TRUE
 ********************************************************************************/
gboolean on_TimeoutPoll_event (gpointer user_data) {
  checkMail (user_data, 0);
  return TRUE;
}

/********************************************************************************
 * Function: Callback on popup timeout
 * Input   : user_data which is a biff structure
 * Output  : Remove the timer, hide the popup and set a new poll timer if necessary
 * Return  : TRUE
 ********************************************************************************/
gboolean on_TimeoutPop_event (gpointer user_data) {
  popdown (user_data);
  return FALSE;
}



/* Generic dialog callbacks */
/********************************************************************************
 * Function: Callback on delete event on a dialog
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Hide all dialogs
 * Return  : TRUE
 ********************************************************************************/
gboolean on_Dialog_delete_event (GtkObject *object, GdkEvent *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  gtk_widget_hide (WIDGET ("FileSelectionDialog"));
  gtk_widget_hide (WIDGET ("FontSelectionDialog"));
  gtk_widget_hide (WIDGET ("ColorSelectionDialog"));
  gtk_main_quit ();
  return TRUE;
}

/********************************************************************************
 * Function: Callback on destroy event on a dialog
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Exit gbiff2
 * Return  : /
 ********************************************************************************/
gboolean on_Dialog_destroy_event (GtkObject *object, GdkEvent *event, gpointer user_data) {
  g_error ("Shazam !\n");
  exit (1);
}

/********************************************************************************
 * Function: Callback on Ok buton clicked on a dialog
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Hide all dialogs, set biff->action to 1
 * Return  : /
 ********************************************************************************/
void on_DialogOkButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  gtk_widget_hide (WIDGET ("FileSelectionDialog"));
  gtk_widget_hide (WIDGET ("FontSelectionDialog"));
  gtk_widget_hide (WIDGET ("ColorSelectionDialog"));
  biff->action = 1;
  gtk_main_quit ();  
}

/********************************************************************************
 * Function: Callback on Ok buton clicked on a dialog
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Hide all dialogs
 * Return  : /
 ********************************************************************************/
void on_DialogCancelButton_clicked (GtkButton *button, gpointer user_data) {
  Biff *biff = (Biff *) user_data;
  gtk_widget_hide (WIDGET ("FileSelectionDialog"));
  gtk_widget_hide (WIDGET ("FontSelectionDialog"));
  gtk_widget_hide (WIDGET ("ColorSelectionDialog"));
  gtk_main_quit ();
}


#ifdef USE_GNOME
/* Applet */
/********************************************************************************
 * Function: Callback on panel size change
 * Input   : GNOME stuff + user_data which is a biff structure
 * Output  : Change applet look according to panel size
 * Return  : /
 ********************************************************************************/
void on_Applet_change_size (GtkWidget *w, int size, gpointer user_data) {
  updateApplet (user_data);
}

/********************************************************************************
 * Function: Callback on panel orientation change
 * Input   : GNOME stuff + user_data which is a biff structure
 * Output  : Change applet look according to panel orientation
 * Return  : /
 ********************************************************************************/
void on_Applet_change_orient (GtkWidget *w, PanelAppletOrient orient, gpointer user_data) {
  updateApplet (user_data);
}

/********************************************************************************
 * Function: Callback on Properties menu
 * Input   : GNOME stuff + user_data which is a biff structure
 * Output  : Raise configuration dialopg and stop polling
 * Return  : /
 ********************************************************************************/
void on_AppletProperties_menu (BonoboUIComponent *uic, gpointer data, const gchar *verbname) {
  Biff *biff = (Biff *) data;

  gtk_widget_hide (biff->popup);
  gtk_widget_show (WIDGET ("ConfigurationDialog"));
  if (biff->polltag > 0) {
    gtk_timeout_remove (biff->polltag);
    biff->polltag = 0;
    if (biff->debug)
      printf ("on_AppletProperties_menu: removing poll timer\n");
  }
  if (biff->poptag > 0) {
    gtk_timeout_remove (biff->poptag);
    biff->poptag = 0;
  };
}


/********************************************************************************
 * Function: Callback on Launch mail reader menu
 * Input   : GNOME stuff + user_data which is a biff structure
 * Output  : Launch mail application
 * Return  : /
 ********************************************************************************/
void on_AppletMailApp_menu (BonoboUIComponent *uic, gpointer data, const gchar *verbname) {
  Biff *biff = (Biff *) data;
  gchar command[2048];

  /* Do not remove the 1024, it prevents buffer overflow */
  sprintf (command, "%.1024s &", biff->mailApp);
  system (command);
}

/********************************************************************************
 * Function: Callback on About menu
 * Input   : GNOME stuff + user_data which is a biff structure
 * Output  : Raise the about dialog
 * Return  : /
 ********************************************************************************/
void on_AppletAbout_menu (BonoboUIComponent *uic, gpointer data, const gchar *verbname) {
  static GtkWidget *about = NULL;
  static const gchar *authors[] = {
    "Nicolas Rougier <Nicolas.Rougier@loria.fr>",
    NULL
  };
 
  if (about != NULL) {
    gdk_window_show (about->window);
    gdk_window_raise (about->window);
    return;
  }
  about = gnome_about_new ("gbiff2 applet", VERSION,
			   "(C) 2000-2002\n ",
			   _("Released under the GNU general public license.\n"\
			   "Checks mail periodically.\n Displays headers when new mail has arrived."),
			   authors,
			   NULL, NULL, gdk_pixbuf_new_from_file (GBIFF2_DATADIR"/gbiff2_logo.png", 0));
  g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK (gtk_widget_destroyed), &about);
  gtk_widget_show (about);
}


/********************************************************************************
 * Function: Callback on applet button 1 press => force mail check
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : /
 * Return  : FALSE
 ********************************************************************************/
gboolean on_Applet_button_press (GtkWidget *w, GdkEventButton *event, gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  if (event->button == 1) {
    if (biff->poptag > 0)
      gtk_timeout_remove (biff->poptag);
    biff->poptag = 0;
    if (biff->polltag > 0)
      gtk_timeout_remove (biff->polltag);
    biff->polltag = 0;
    if (biff->debug)
      printf ("on_Applet_button_press: removing poll timer\n");


    /* If the popup was on display, then the event is a request to hide it */
    if (GTK_WIDGET_VISIBLE(biff->popup))
      popdown (biff);
    else
      checkMail (biff, 1);
  }	
  
  return FALSE;
}

/********************************************************************************
 * Function: Callback on applet button release
 * Input   : GTK stuff + user_data which is a biff structure
 * Output  : Set pop timer if popup is on
 * Return  : FALSE
 ********************************************************************************/
gboolean on_Applet_button_release (GtkWidget *widget, GdkEventButton *event,  gpointer user_data) {
  Biff *biff = (Biff *) user_data;

  if (GTK_WIDGET_VISIBLE (biff->popup)) {
    if (biff->poptag > 0)
      gtk_timeout_remove (biff->poptag);
    biff->poptag = gtk_timeout_add (biff->poptime*1000, on_TimeoutPop_event, biff);
  }

  return FALSE;
}

#endif /* ifdef USE_GNOME */
