/*
 * gbiff2 program
 * Copyright (C) 2000-2002 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifdef USE_GNOME
#  include <gnome.h>
#  include <panel-applet.h>
#  include <panel-applet-gconf.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <glade/glade-build.h>

#ifndef __GBIFF2_H
#define __GBIFF2_H

/************
 * Constants
 ************/
#define PROTOCOL_FILE			0
#define PROTOCOL_POP3			1
#define PROTOCOL_IMAP4			2
#define PROTOCOL_MAILDIR		3
#define PROTOCOL_MH			4
#define PROTOCOL_APOP			5
#define DEFAULT_POP3_PORT		110
#define DEFAULT_IMAP4_PORT		143

#define	NO_SOUND			0
#define SYSTEM_BEEP			1
#define SOUNDFILE			2

#define STATUS_NEW_MAIL			0
#define STATUS_OLD_MAIL			1
#define STATUS_NO_MAIL			2
#define STATUS_ERROR			3

#define AUTOMATIC_MODE			0
#define MANUAL_MODE			1

#define MAX_HEADERS			100


/*******************
 * Default settings
 *******************/
#define DEFAULT_TITLE			"Mail"
#define DEFAULT_PROTOCOL		PROTOCOL_FILE
#define	DEFAULT_POLLTIME		60
#define	DEFAULT_POPTIME			5
#define DEFAULT_POPUP_GEOMETRY		"-0+0"
#define DEFAULT_POPUP_Y			0
#define DEFAULT_POPUP_DECORATED		TRUE
#define DEFAULT_POPUP_STICKY		FALSE
#define DEFAULT_BIFF_GEOMETRY		"+0+0"
#define DEFAULT_BIFF_DECORATED		TRUE
#define DEFAULT_BIFF_STICKY		FALSE
#define DEFAULT_DISPLAY_POPUP		TRUE
#define DEFAULT_DISPLAY_TITLE		TRUE
#define DEFAULT_DISPLAY_DATE		TRUE
#define DEFAULT_DISPLAY_READMAIL	TRUE
#define DEFAULT_FROM_SIZE		50
#define DEFAULT_SUBJECT_SIZE		50
#define DEFAULT_SOUND_TYPE		2
#define DEFAULT_MAILAPP			"emacs"
#define DEFAULT_SOUNDFILE		REAL_DATADIR"/sounds/gbiff2/mail.wav"
#define DEFAULT_SOUND_VOLUME		25
#define DEFAULT_TITLE_FONT		"Sans 12"
#define DEFAULT_BODY_FONT		"Sans 10"
#define DEFAULT_BIFF_FONT		"Sans 8"
#define DEFAULT_UNREAD_COLOR		"blue"
#define DEFAULT_READ_COLOR		"black"
#ifdef USE_GNOME
#define DEFAULT_MAIL_IMAGE_FILE		GBIFF2_DATADIR"/gnome-mail.xpm"
#define DEFAULT_NOMAIL_IMAGE_FILE	GBIFF2_DATADIR"/gnome-nomail.xpm"
#else
#define DEFAULT_MAIL_IMAGE_FILE		GBIFF2_DATADIR"/gtk-mail.xpm"
#define DEFAULT_NOMAIL_IMAGE_FILE	GBIFF2_DATADIR"/gtk-nomail.xpm"
#endif
#define DEFAULT_GLADE_FILE		GBIFF2_DATADIR"/gbiff2.glade2"
#define DEFAULT_CONFIG_FILE		".gbiff2rc"


/******************************
 * Header structure definition
 ******************************/
typedef struct _header {
  gchar	*	sender;			/* Sender of the mail			*/
  gchar *	subject;		/* Subject of the mail			*/
  gchar *	date;			/* Date of mail sending			*/
  int		status;			/* Status of the mail			*/
  gchar *	body;			/* Body of the mail (first n bytes)	*/
  gchar *	extra;			/* Extra informations			*/
} Header;


/****************************
 * Biff structure definition
 ****************************/
typedef struct _biff {
  char *	title;			/* Title of the mailbox			*/
  int		protocol;		/* Protocol to use			*/
  gchar *	address;		/* Address of the mailbox		*/
  gchar *	user;			/* Username (for networked mailboxes)	*/
  gchar *	password;		/* Password (for networked mailboxes)	*/
  int		polltime;		/* Time between polls			*/
  int		poptime;		/* Popup display duration		*/

  gchar *	popupGeometry;		/* Popup geometry			*/
  int		popupDecorated;		/* Popup decoration 			*/
  int		popupSticky;		/* Popup stick property			*/
  gchar *	biffGeometry;		/* Biff geometry (GTK only)		*/
  int		biffDecorated;		/* Biff decoration(GTK only)		*/
  int		biffSticky;		/* Biff stick property (GTK only)	*/

  int		displayPopup;		/* Display popup or not			*/
  int		displayTitle;		/* Display title or not in popup	*/
  int		displayDate;		/* Display data or not in popup		*/
  int		displayReadMail;	/* Display read mail or nor in popup	*/
  int		fromSize;		/* Maximum size of the 'From' field	*/
  int		subjectSize;		/* Maximum size of the 'Subject' field	*/
  int		soundType;		/* Sound type : none,
					                system beep,
							or soundfile		*/
  int		soundVolume;		/* Sound volume				*/
  gchar *	mailApp;		/* Mail app. to launch if requested	*/
  gchar *	soundFile;		/* Sound file to play			*/
  gchar *	titleFont;		/* Font to use for popup title		*/
  gchar *	bodyFont;		/* Font to use for popup body		*/
  gchar *	biffFont;		/* Font to use in
					   biff window (GTK) or applet (GNOME)	*/
  gchar *	readColor;		/* Color to use for read mails		*/
  gchar *	unreadColor;		/* Color to use for unread mails	*/
  gchar *	mailImageFile;		/* Image to use when new mail		*/
  gchar *	nomailImageFile;	/* Image to use when no new mail	*/

  gchar *	gladeFile;		/* Interface file (glade)		*/
  gchar *	configFile;		/* Configuration file			*/

  Header	*headers[MAX_HEADERS];	/* Headers				*/
  int		headerSize;		/* Total headers size (unread+read)	*/
  int 		unread;			/* Unread mail				*/
  int 		read;			/* Read mail				*/
  int		status;			/* Biff status				*/
  
  int 		pollMode;		/* Automatic or manual mode		*/
  int		polltag;		/* Timer tag for poll timer		*/
  int		poptag;			/* Timer tag for pop timer		*/

  int		noconfigure;		/* Configure at start or not		*/
  int		debug;			/* Debug output or not			*/
  int		block;			/* Block flag for blocking callbacks	*/
  int		action;			/* Action flag for dialog
					        (0 for cancel, 1 for OK)	*/

  GladeXML *	xml;			/* XML glade parser			*/
  GtkWidget *	biff;			/* Biff applet (GNOME) or window (GTK)	*/
  GtkWidget *	biffTable;		/* Biff table (GNOME)			*/
  GtkWidget *	biffDA;			/* Biff drawing area (GTK)		*/
  GdkGC *	biffGC;			/* Biff graphic context			*/
  GtkWidget *	popup;			/* Popup dialog				*/
  GtkWidget *	popupTreeView;		/* Popup tree view			*/
} Biff;


/*************
 * Prototypes
 *************/
int main (int argc, char **argv);
int loadConfig (Biff *biff);
int saveConfig (Biff *biff);

#endif /* __GBIFF2_H */
