/*
 * gbiff2 program
 * Copyright (C) 2000-2002 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gbiff2.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "protocol.h"


/********************************************************************************
 * Function: Build the interface using libglade 2 and other buildXXX functions
 * Input   : a biff structure
 * Output  : biff->xml
 * Return  : Error code (0 if OK, 1 else)
 * Note    : /
 ********************************************************************************/
int buildInterface (Biff *biff) {
  /* Glade parsing of the interface */
  biff->xml = glade_xml_new (biff->gladeFile, NULL, NULL);

  /* If we got an error, we show an error popup and we exit */
  if (!biff->xml) {
    GtkWidget *dialog =
      gtk_message_dialog_new (NULL,
			      0,
			      GTK_MESSAGE_ERROR,
			      GTK_BUTTONS_OK,
			      _("gbiff2 could not load its user interface file (%s).\n"
			        "Please make sure gbiff2 was installed correctly."),
			      biff->gladeFile);
    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
    return 1;
  }
  glade_xml_signal_autoconnect_full (biff->xml, MyGladeXMLConnectFunc, biff);
  gtk_widget_modify_font (WIDGET ("ThanksText"), getfont ("Sans 9"));

  biff->popup = WIDGET ("PopupDialog");

  /* Self made widgets */
  buildPopup (biff);

#ifndef USE_GNOME
  biff->biff = WIDGET ("BiffDialog");
  biff->biffDA = WIDGET ("BiffDA");
  gtk_widget_set_events (biff->biffDA, GDK_ALL_EVENTS_MASK);
#endif

  /* Update interface */
  updateBiffToInterface (biff);

  return 0;
}

/********************************************************************************
 * Function: Build the popup window
 * Input   : a biff structure
 * Output  : popupTreeView
 * Return  : /
 ********************************************************************************/
void buildPopup (Biff *biff) {
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;
  GtkListStore *store;
  GtkTreeModel *model;

  /* Popup */
  biff->popup = WIDGET("PopupDialog");

  /* List */
  store = gtk_list_store_new (5,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING);
  model = GTK_TREE_MODEL (store);
  biff->popupTreeView = gtk_tree_view_new_with_model (model);
  gtk_widget_set_events (biff->popupTreeView,
			 GDK_ENTER_NOTIFY_MASK |
			 GDK_LEAVE_NOTIFY_MASK |
			 GDK_BUTTON_PRESS_MASK);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (biff->popupTreeView), TRUE);
  gtk_tree_view_set_search_column (GTK_TREE_VIEW (biff->popupTreeView), 0);
  gtk_tree_view_columns_autosize (GTK_TREE_VIEW (biff->popupTreeView));

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("#", renderer, "text", 0, "foreground", 4, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (biff->popupTreeView), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("From"), renderer, "text", 1, "foreground", 4, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (biff->popupTreeView), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Subject"), renderer, "text", 2,"foreground", 4,  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (biff->popupTreeView), column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer, "text", 3, "foreground", 4, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (biff->popupTreeView), column);
  
  gtk_box_pack_start (GTK_BOX(WIDGET ("PopupVBox")), biff->popupTreeView, TRUE, TRUE, 0);
  g_object_unref (G_OBJECT (model));
  gtk_widget_show (biff->popupTreeView);


  /* Callbacks */
  g_signal_connect (G_OBJECT (biff->popupTreeView), "button_press_event",
		    GTK_SIGNAL_FUNC (on_Popup_button_press_event),
		    biff);
  g_signal_connect (G_OBJECT (biff->popupTreeView), "enter_notify_event",
		    GTK_SIGNAL_FUNC (on_Popup_enter_notify_event),
		    biff);
  g_signal_connect (G_OBJECT (biff->popupTreeView), "leave_notify_event",
	 	    GTK_SIGNAL_FUNC (on_Popup_leave_notify_event),
		    biff);
}


/********************************************************************************
 * Function: Build the applet (GNOME)
 * Input   : a biff structure
 * Output  : biff->biff
 * Return  : TRUE
 ********************************************************************************/
#ifdef USE_GNOME
int buildApplet (GtkWidget *applet, Biff *biff) {
  int i;
  static const BonoboUIVerb gbiff2MenuVerbs [] = {
    BONOBO_UI_VERB ("Props",   on_AppletProperties_menu),
    BONOBO_UI_VERB ("MailApp", on_AppletMailApp_menu),
    BONOBO_UI_VERB ("About",   on_AppletAbout_menu), 
    BONOBO_UI_VERB_END
  };
 
  /* Allocate headers in biff */
  for (i=0; i<MAX_HEADERS; i++)
    biff->headers[i] = g_malloc (sizeof (Header));
  /* Get the configuration file */
  biff->configFile = g_strconcat ((gchar *) g_getenv("HOME"), "/.gbiff2rc", NULL);
  /* Get default address */
  biff->address = g_strdup ((gchar *) g_getenv ("MAIL"));
  /* Get default username */
  biff->user = g_strdup ((gchar *) g_getenv ("LOGNAME"));
  /* Set the debug flag if DEBUG is defined */
#ifdef DEBUG
  biff->debug = 1;
#endif
  /* Load the config file */
  loadConfig (biff);

  /* Interface creation */
  if (buildInterface (biff) == 1)
    return FALSE;
  
//  panel_applet_add_preferences (applet, "/schemas/apps/gbiff2/prefs", NULL);
  panel_applet_setup_menu_from_file (PANEL_APPLET (applet),
				     NULL,
				     "GNOME_gbiff2Applet.xml",
				     NULL,
				     gbiff2MenuVerbs,
				     biff);

  /* Put the applet table into applet */
  biff->biff = applet;
  gtk_widget_reparent (WIDGET ("AppletTable"), applet);  

  /* Signals */
  g_signal_connect (G_OBJECT (applet),"change_orient",
		    GTK_SIGNAL_FUNC (on_Applet_change_orient),
		    biff);
  g_signal_connect (G_OBJECT (applet),"change_size",
		    GTK_SIGNAL_FUNC (on_Applet_change_size),
		    biff);
  g_signal_connect (G_OBJECT (applet), "button_press_event",
		    GTK_SIGNAL_FUNC (on_Applet_button_press),
		    biff);
  g_signal_connect (G_OBJECT (applet), "button_release_event",
		    GTK_SIGNAL_FUNC (on_Applet_button_release),
		    biff);

  if (biff->pollMode == AUTOMATIC_MODE) {
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      if (biff->debug)
	printf ("buildApplet: removing poll timer\n");
    }
    biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
      if (biff->debug)
	printf ("buildApplet: adding poll timer (%d)\n", biff->polltime);
  }

  gtk_widget_show (applet);
  updateApplet (biff);

  return TRUE;
}
#endif /* ifdef USE_GNOME */

/********************************************************************************
 * Function: Update widgets according to biff
 * Input   : a biff structure
 * Output  : all widgets
 * Return  : /
 * Note    : /
 ********************************************************************************/
void updateBiffToInterface (Biff *biff) {
  int flag;
  GdkColor color;
  gchar *buffer;
  static GdkPixbuf *mailpix = 0;
  static GdkPixbuf *nomailpix = 0;

  /* Flag for blocking handler */
  biff->block = 1;

  /* POPUP */
  /* Title */
  gtk_label_set_text (GTK_LABEL (WIDGET ("PopupTitle")), locale_to_utf8 (biff->title));
  gtk_window_set_title (GTK_WINDOW(biff->popup), locale_to_utf8 (biff->title));
  gtk_widget_modify_font (GTK_WIDGET (WIDGET ("PopupTitle")), getfont (biff->titleFont));
  if ((biff->displayTitle == 0) || (biff->popupDecorated == 1))
    gtk_widget_hide (GTK_WIDGET (WIDGET ("PopupTitle")));
  else
    gtk_widget_show (GTK_WIDGET (WIDGET ("PopupTitle")));


  /* List */
  gtk_widget_modify_font (GTK_WIDGET (biff->popupTreeView), getfont (biff->bodyFont));
  if (gtk_tree_view_get_column (GTK_TREE_VIEW (biff->popupTreeView), 3))
    gtk_tree_view_remove_column (GTK_TREE_VIEW (biff->popupTreeView),
				 gtk_tree_view_get_column (GTK_TREE_VIEW (biff->popupTreeView), 3));
  if (biff->displayDate == 1) {
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
    GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer, "text", 3, "foreground", 4, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (biff->popupTreeView), column);
  }
  /* END POPUP */


  /* CONFIGURATION DIALOG */
  /* Title */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("TitleEntry")), locale_to_utf8 (biff->title));
  
  /* Protocol */
  if (biff->protocol == PROTOCOL_MH) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioMh")), TRUE);
    flag = FALSE;
  }
  else if (biff->protocol == PROTOCOL_MAILDIR) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioMaildir")), TRUE);
    flag = FALSE;
  }
  else if (biff->protocol == PROTOCOL_POP3) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioPop3")), TRUE);
    flag = TRUE;
  }
  else if (biff->protocol == PROTOCOL_IMAP4) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioImap4")), TRUE);
    flag = TRUE;
  }
  else if (biff->protocol == PROTOCOL_APOP) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioApop")), TRUE);
    flag = TRUE;
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioFile")), TRUE);
    flag = FALSE;
  }
  gtk_widget_set_sensitive (WIDGET ("UserEntry"), flag);
  gtk_widget_set_sensitive (WIDGET ("UserLabel"), flag);
  gtk_widget_set_sensitive (WIDGET ("PasswordEntry"), flag);
  gtk_widget_set_sensitive (WIDGET ("PasswordLabel"), flag);

  /* Address */
  if ((biff->protocol == PROTOCOL_POP3) || (biff->protocol == PROTOCOL_APOP) || (biff->protocol == PROTOCOL_IMAP4)) {
    if (strchr (biff->address, (int) ':') == 0) {
      buffer = g_strconcat (biff->address, ":0000", NULL);
      if ((biff->protocol == PROTOCOL_POP3) || (biff->protocol == PROTOCOL_APOP))
	sprintf (buffer, "%s:%.4d", biff->address, DEFAULT_POP3_PORT);
      if (biff->protocol == PROTOCOL_IMAP4)
	sprintf (buffer, "%s:%.4d", biff->address, DEFAULT_IMAP4_PORT);
      g_free (biff->address);
      biff->address = buffer;
    }
  }
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("AddressEntry")), filename_to_utf8 (biff->address));

  /* User */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("UserEntry")), locale_to_utf8 (biff->user));

  /* Password */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("PasswordEntry")), locale_to_utf8 (biff->password));

  /* Polltime spin */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (WIDGET ("PollTimeSpin")), biff->polltime);

  /* Poptime spin */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (WIDGET ("PopTimeSpin")), biff->poptime);

  /* Popup geometry */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("PopupGeometryEntry")), locale_to_utf8 (biff->popupGeometry));

  /* Popup sticky */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("PopupStickyCheck")), biff->popupSticky);
  //  if (biff->popupSticky)
  //    gtk_window_stick (GTK_WINDOW (biff->popup));
  //  else
  //    gtk_window_unstick (GTK_WINDOW (biff->popup));


  /* Popup decorated */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("PopupDecoratedCheck")), biff->popupDecorated);
  if (gtk_window_get_decorated (GTK_WINDOW (biff->popup)) != biff->popupDecorated) {
    if (GTK_WIDGET_VISIBLE (biff->popup)) {
      gtk_widget_hide (biff->popup);
      gtk_window_set_decorated (GTK_WINDOW (biff->popup), biff->popupDecorated);
      gtk_widget_show (biff->popup);
    }
    else
      gtk_window_set_decorated (GTK_WINDOW (biff->popup), biff->popupDecorated);
  }


#ifndef USE_GNOME
  /* Biff geometry */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("BiffGeometryEntry")), locale_to_utf8 (biff->biffGeometry));

  /* Biff decorated */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("BiffDecoratedCheck")), biff->biffDecorated);
  if (gtk_window_get_decorated (GTK_WINDOW (biff->biff)) != biff->biffDecorated) {
    if (GTK_WIDGET_VISIBLE (biff->biff)) {
      gtk_widget_hide (biff->biff);
      gtk_window_set_decorated (GTK_WINDOW (biff->biff), biff->biffDecorated);
      gtk_widget_show (biff->biff);
    }
    else
      gtk_window_set_decorated (GTK_WINDOW (biff->biff), biff->biffDecorated);
  }

  /* Biff sticky */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("BiffStickyCheck")), biff->biffSticky);
  //  if (biff->biffSticky)
  //    gtk_window_stick (GTK_WINDOW (biff->biff));
  //  else
  //    gtk_window_unstick (GTK_WINDOW (biff->biff));
#else
  gtk_widget_set_sensitive (WIDGET ("BiffGeometryTable"), FALSE);
#endif


  /* Show popup */
  flag = biff->displayPopup;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayPopupCheck")), flag);
  gtk_widget_set_sensitive (WIDGET ("DisplayTitleCheck"), flag);
  gtk_widget_set_sensitive (WIDGET ("DisplayDateCheck"), flag);
  gtk_widget_set_sensitive (WIDGET ("DisplayReadMailCheck"), flag);
  gtk_widget_set_sensitive (WIDGET ("PopupTitleButton"), flag);
  gtk_widget_set_sensitive (WIDGET ("PopupBodyButton"), flag);
  gtk_widget_set_sensitive (WIDGET ("PopupGeometryTable"), flag);
  gtk_widget_set_sensitive (WIDGET ("FromFieldHBox"), flag);
  gtk_widget_set_sensitive (WIDGET ("SubjectFieldHBox"), flag);
  gtk_widget_set_sensitive (WIDGET ("ColorVBox"), flag);

  /* Show title */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayTitleCheck")), biff->displayTitle);

  /* Show date */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayDateCheck")), biff->displayDate);

  /* Show read mail */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayReadMailCheck")), biff->displayReadMail);
  flag = biff->displayReadMail;
  gtk_widget_set_sensitive (WIDGET ("ReadEventBox"), flag);
  gtk_widget_set_sensitive (WIDGET ("ReadLabel"), flag);
  
  /* 'From' size */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (WIDGET ("FromFieldSpin")), (gdouble) biff->fromSize);

  /* 'Subject' size */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (WIDGET ("SubjectFieldSpin")), (gdouble) biff->subjectSize);
  

  /* Mail application */
  gtk_entry_set_text (GTK_ENTRY (WIDGET ("MailAppEntry")), filename_to_utf8 (biff->mailApp));

  /* Sound type */
  if (biff->soundType == 2) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("SoundPlayCheck")), TRUE);
    flag = TRUE;
  }
  else if (biff->soundType == 1) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("SystemBeepCheck")), TRUE);
    flag = FALSE;
  }
  else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WIDGET ("NoSoundCheck")), TRUE);
    flag = FALSE;
  }
  gtk_widget_set_sensitive (WIDGET ("SoundLabel"), flag);
  gtk_widget_set_sensitive (WIDGET ("SoundButton"), flag);
  gtk_widget_set_sensitive (WIDGET ("SoundPlayButton"), flag);
  gtk_widget_set_sensitive (WIDGET ("VolumeLabel"), flag);
  gtk_widget_set_sensitive (WIDGET ("VolumeHScale"), flag);

  /* Sound button label */
  gtk_label_set_text (GTK_LABEL(GTK_BIN(WIDGET ("SoundButton"))->child),
		      filename_to_utf8 ((gchar *)(strrchr(biff->soundFile, '/')+1)));

  /* Sound volume */
  gtk_range_set_value (GTK_RANGE (WIDGET ("VolumeHScale")), biff->soundVolume);

  /* Font buttons */
  gtk_widget_modify_font (GTK_BIN(WIDGET ("PopupTitleButton"))->child, getfont (biff->titleFont));
  gtk_widget_modify_font (GTK_BIN(WIDGET ("PopupBodyButton"))->child, getfont (biff->bodyFont));
  gtk_widget_modify_font (GTK_BIN(WIDGET ("BiffButton"))->child, getfont (biff->biffFont));
  gtk_widget_modify_font (WIDGET ("AppletTitleLabel"), getfont (biff->biffFont));
  gtk_label_set_text (GTK_LABEL (WIDGET ("AppletTitleLabel")), locale_to_utf8 (biff->title));
  gtk_widget_modify_font (WIDGET ("AppletHUnreadLabel"), getfont (biff->biffFont));
  gtk_widget_modify_font (WIDGET ("AppletVUnreadLabel"), getfont (biff->biffFont));

/*
  if (biff->displayTitle == 0)
    gtk_widget_hide (WIDGET ("AppletTitleLabel"));
  else
    gtk_widget_show (WIDGET ("AppletTitleLabel"));
*/


  /* Mail image */
  if (mailpix != 0)
    g_object_unref (mailpix);
  mailpix = gdk_pixbuf_new_from_file (biff->mailImageFile, 0);
  gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("MailImage")), mailpix);
# ifndef USE_GNOME
  gtk_widget_set_size_request (biff->biffDA, gdk_pixbuf_get_width (mailpix), gdk_pixbuf_get_height (mailpix));
# endif

  /* No mail image */
  if (nomailpix != 0)
    g_object_unref (nomailpix);
  nomailpix = gdk_pixbuf_new_from_file (biff->nomailImageFile, 0);
  gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("NoMailImage")), nomailpix);

  /* Unread mail color */
  gdk_color_parse (biff->unreadColor, &color);
  gtk_widget_modify_fg  (WIDGET("UnreadLabel"), GTK_STATE_NORMAL, &color);
  gdk_color_parse ("#ffffff", &color);
  gtk_widget_modify_bg  (WIDGET("UnreadEventBox"), GTK_STATE_NORMAL, &color);

  /* Read mail color */
  gdk_color_parse (biff->readColor, &color);
  gtk_widget_modify_fg  (WIDGET("ReadLabel"), GTK_STATE_NORMAL, &color);
  gdk_color_parse ("#eeeeee", &color);
  gtk_widget_modify_bg  (WIDGET("ReadEventBox"), GTK_STATE_NORMAL, &color);
  /* END CONFIGURATION DIALOG */

  biff->block = 0;
}


/********************************************************************************
 * Function: Update biff structure according to widgets
 * Input   : a biff structure
 * Output  : biff structure
 * Return  : /
 ********************************************************************************/
void updateBiffFromInterface (Biff *biff) {
  gchar *buffer;

  /* Title */
  g_free (biff->title);
  biff->title = g_strdup (locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("TitleEntry")))));

  /* Protocol */
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioMh"))))
    biff->protocol = PROTOCOL_MH;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioMaildir"))))
    biff->protocol = PROTOCOL_MAILDIR;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioPop3"))))
    biff->protocol = PROTOCOL_POP3;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioApop"))))
    biff->protocol = PROTOCOL_APOP;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("ProtocolRadioImap4"))))
    biff->protocol = PROTOCOL_IMAP4;
  else
    biff->protocol = PROTOCOL_FILE;

  /* Address */
  g_free (biff->address);
  biff->address = g_strdup (filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("AddressEntry")))));
  if ((biff->protocol == PROTOCOL_POP3) || (biff->protocol == PROTOCOL_APOP) || (biff->protocol == PROTOCOL_IMAP4)) {
    if (strchr (biff->address, (int) ':') == 0) {
      buffer = g_strconcat (biff->address, ":0000", NULL);
      if ((biff->protocol == PROTOCOL_POP3)|| (biff->protocol == PROTOCOL_APOP))
	sprintf (buffer, "%s:%.4d", biff->address, DEFAULT_POP3_PORT);
      if (biff->protocol == PROTOCOL_IMAP4)
	sprintf (buffer, "%s:%.4d", biff->address, DEFAULT_IMAP4_PORT);
      g_free (biff->address);
      biff->address = buffer;
    }
  }

  /* User */
  g_free (biff->user);
  biff->user = g_strdup (locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("UserEntry")))));

  /* Password */
  g_free (biff->password);
  biff->password =  g_strdup (locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("PasswordEntry")))));

  /* Polltime spin */
  biff->polltime = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (WIDGET ("PollTimeSpin")));

  /* Poptime spin */
  biff->poptime = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (WIDGET ("PopTimeSpin")));

  /* Popup geometry */
  g_free (biff->popupGeometry);
  biff->popupGeometry = g_strdup (locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("PopupGeometryEntry")))));

  /* Popup decorated */
  biff->popupDecorated =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("PopupDecoratedCheck")));

  /* Popup sticky */
  biff->popupSticky =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("PopupStickyCheck")));

#ifndef USE_GNOME
  /* Popup geometry */
  g_free (biff->biffGeometry);
  biff->biffGeometry = g_strdup (locale_from_utf8 (gtk_entry_get_text (GTK_ENTRY (WIDGET ("BiffGeometryEntry")))));
  /* Biff decorated */
  biff->biffDecorated =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("BiffDecoratedCheck")));
  /* Biff sticky */
  biff->biffSticky =  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("BiffStickyCheck")));
#endif
  
  /* Show popup */
  biff->displayPopup = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayPopupCheck")));

  /* Show title */
  biff->displayTitle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayTitleCheck")));

  /* Show date */
  biff->displayDate = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayDateCheck")));

  /* Show read mail */
  biff->displayReadMail = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("DisplayReadMailCheck")));

  /* 'From' size */
  biff->fromSize = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (WIDGET ("FromFieldSpin")));

  /* 'Subject' size */
  biff->subjectSize = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (WIDGET ("SubjectFieldSpin")));

  /* Sound type */
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("SoundPlayCheck"))))
    biff->soundType = 2;
  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WIDGET ("SystemBeepCheck"))))
    biff->soundType = 1;
  else
    biff->soundType = 0;

  /* Sound volume */
  biff->soundVolume = (int) (gtk_range_get_value (GTK_RANGE (WIDGET ("VolumeHScale"))));
}

/********************************************************************************
 * Function: Update the biff window according to mail status
 * Input   : A biff structure
 * Output  : biff->biff
 * Return  : /
 ********************************************************************************/
void updateBiff (Biff *biff) {
  GdkPixbuf *gdkpix;
  static char unread[16];
  int width, height, textwidth, textheight;
  PangoLayout *pangolayout;

  /* Create a gc if none available */
  if (biff->biffGC == 0)
    biff->biffGC = gdk_gc_new (biff->biffDA->window);

  /* Set text to be displayed and the right pixmap */
  if (biff->status != STATUS_ERROR) {
    if (biff->unread < MAX_HEADERS) {
      sprintf (unread, _("Unread:%.2d"), biff->unread);
      if (biff->unread > 0)
	gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("MailImage")));
      else
	gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("NoMailImage")));
    }
    else {
      gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("MailImage")));
      sprintf  (unread, _("Unread:++"));
    }
  }
  else {
    gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("NoMailImage")));
    sprintf  (unread, _("Unread:--"));
  };

  /* Get DA size */
  width = gdk_pixbuf_get_width (gdkpix);
  height = gdk_pixbuf_get_height (gdkpix);

  /* Render pixmap */
  gdk_pixbuf_render_to_drawable (gdkpix, biff->biffDA->window, biff->biffGC, 0,0,0,0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);


  /* Title */
  /* Compute text size */
  pangolayout = gtk_widget_create_pango_layout (biff->biffDA, biff->title);
  pango_layout_set_font_description (pangolayout, getfont (biff->biffFont));
  pango_layout_get_size (pangolayout, &textwidth, &textheight);
  /* Render font */
  gdk_draw_layout (biff->biffDA->window, biff->biffGC,
		   (width-textwidth/PANGO_SCALE)/2,
		   0,
		   pangolayout);
  g_object_unref (pangolayout);

  /* Unread */
  /* Compute text size */
  pangolayout = gtk_widget_create_pango_layout (biff->biffDA, unread);
  pango_layout_set_font_description (pangolayout, getfont (biff->biffFont));
  pango_layout_get_size (pangolayout, &textwidth, &textheight);
  /* Render font */
  gdk_draw_layout (biff->biffDA->window, biff->biffGC,
		   (width-textwidth/PANGO_SCALE)/2,
		   height-textheight/PANGO_SCALE,
		   pangolayout);

  g_object_unref (pangolayout);
};

/********************************************************************************
 * Function: Perform a mail checking and update widgets depending on result
 * Input   : A biff structure
 * Output  : biff & screen
 * Return  : /
 ********************************************************************************/
void checkMail (Biff *biff, int force) {
  GtkListStore *store;
  GtkTreeIter iter;
  int i, number;
  char text[16];
  static gchar *command = 0;

  if (command == 0)
    command = g_malloc (4096);

  /* Check for new mail depending on protocol */
  if ((biff->protocol == PROTOCOL_FILE) && (FILE_checkmail(biff) == 0))
    FILE_getheaders (biff);
  else if ((biff->protocol == PROTOCOL_POP3) && (POP3_checkmail(biff) == 0))
    POP3_getheaders (biff);
  else if ((biff->protocol == PROTOCOL_APOP) && (POP3_checkmail(biff) == 0))
    POP3_getheaders (biff);
  else if ((biff->protocol == PROTOCOL_MAILDIR) && (MAILDIR_checkmail(biff) == 0))
    MAILDIR_getheaders (biff);
  else if ((biff->protocol == PROTOCOL_IMAP4) && (IMAP4_checkmail(biff) == 0))
    IMAP4_getheaders (biff);
  else if ((biff->protocol == PROTOCOL_MH) && (MH_checkmail(biff) == 0))
    MH_getheaders (biff);

  if (biff->status != STATUS_ERROR) {
    /* Update header list in popup*/
    store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW ((biff->popupTreeView))));
    gtk_list_store_clear (store);
    number = 1;

    /* First pass for unread mail */
    for (i=0; i<biff->headerSize; i++)
      if (biff->headers[i]->status == 0) {
	sprintf (text, "%.2d", number++);
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter,
			    0, text,
			    1, biff->headers[i]->sender,
			    2, biff->headers[i]->subject,
			    3, biff->headers[i]->date,
			    4, biff->unreadColor,
			    -1);
	}
      /* Second pass for read mail */
      for (i=0; i<biff->headerSize; i++)
	if (biff->headers[i]->status == 1) {
	  sprintf (text, "%.2d", number++);
	  gtk_list_store_append (store, &iter);
	  gtk_list_store_set (store, &iter,
			      0, text, 
	                      1, biff->headers[i]->sender,
	                      2, biff->headers[i]->subject,
			      3, biff->headers[i]->date,
			      4, biff->readColor,
			      -1);
	}
  }

  /* Update biff window (GTK) */
# ifdef USE_GNOME
  updateApplet (biff);
# else
  updateBiff (biff);
# endif

  /* Update popup position */
  if (((biff->status == STATUS_NEW_MAIL) && ((biff->unread > 0) && (biff->displayPopup))) || 
      ((force) && (biff->headerSize > 0))) {

    if (!GDK_IS_WINDOW(GTK_WIDGET (biff->popup)->window))
      gtk_widget_show (biff->popup);
    my_gtk_window_parse_geometry (GTK_WINDOW(biff->popup), biff->popupGeometry);
    gtk_widget_show (biff->popup);
    if (biff->polltag > 0) {
      gtk_timeout_remove (biff->polltag);
      biff->polltag = 0;
      if (biff->debug)
	printf ("checkMail: removing poll timer\n");
    };
    if (biff->poptag > 0)
      gtk_timeout_remove (biff->poptag);
    biff->poptag = gtk_timeout_add (biff->poptime*1000, on_TimeoutPop_event, biff);
  }
  else if (biff->polltag == 0) {
    biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
    if (biff->debug)
      printf ("checkMail: adding poll timer (%d)\n", biff->polltime);
  }

  /* Make sound */
  if ((!force) && (biff->soundType == 1) && (biff->status == STATUS_NEW_MAIL))
    gdk_beep ();
  else if ((!force) && (biff->soundType == 2) && (biff->status == STATUS_NEW_MAIL)) {
    /* Do not remove the 1024, it prevents buffer overflow */
    sprintf (command, "play %.4080s -v %f&", biff->soundFile, biff->soundVolume/100.0);
    system (command);
  }
}

/********************************************************************************
 * Function: Pop the popup down and save its position
 * Input   : A biff structure
 * Output  : biff & screen
 * Return  : /
 ********************************************************************************/
void popdown (Biff *biff) {
  /* Remove pop timer */
  if (biff->poptag > 0) {
    gtk_timeout_remove (biff->poptag);
    biff->poptag = 0;
  };

  /* Add a new poll timer if necessary */
  if (biff->pollMode == AUTOMATIC_MODE) {
    if (biff->polltag == 0) {
      biff->polltag = gtk_timeout_add (biff->polltime*1000, on_TimeoutPoll_event, biff);
      if (biff->debug)
	printf ("popdown: adding poll timer (%d)\n", biff->polltime);
    }
  }
  /* Hide the popup */
  gtk_widget_hide (biff->popup);
}


/********************************************************************************
 * Function: Update the applet according to mail status (GNOME)
 * Input   : A biff structure
 * Output  : biff->biff
 * Return  : /
 ********************************************************************************/
#ifdef USE_GNOME
void updateApplet (Biff *biff) {
  GdkPixbuf *gdkpix;
  static char unread[16];
  guint size;
  PanelAppletOrient orient;

  /* Set text to be displayed and the right pixmap */
  if (biff->status != STATUS_ERROR) {
    if (biff->unread < MAX_HEADERS) {
      sprintf (unread, "Unread:%.2d", biff->unread);
      if (biff->unread > 0)
	gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("MailImage")));
      else
	gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("NoMailImage")));
    }
    else {
      gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("MailImage")));
      sprintf  (unread, "Unread:++");
    }
  }
  else {
    gdkpix = gtk_image_get_pixbuf (GTK_IMAGE(WIDGET("NoMailImage")));
    sprintf  (unread, "Unread:--");
  };

  gtk_label_set_text (GTK_LABEL(WIDGET ("AppletHUnreadLabel")), locale_to_utf8 (unread));
  gtk_label_set_text (GTK_LABEL(WIDGET ("AppletVUnreadLabel")), locale_to_utf8 (unread));
  gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("AppletHImage")), gdkpix);
  gtk_image_set_from_pixbuf (GTK_IMAGE (WIDGET("AppletVImage")), gdkpix);

  size = panel_applet_get_size (PANEL_APPLET (biff->biff));
  orient = panel_applet_get_orient (PANEL_APPLET (biff->biff));
  
  /* The panel is oriented horizontally */
  if ((orient == PANEL_APPLET_ORIENT_DOWN) || (orient == PANEL_APPLET_ORIENT_UP)) {
    /* For size <= 24, we display only the image */
    if (size <= 24) {
      gtk_widget_hide (WIDGET ("AppletVUnreadLabel"));
      gtk_widget_hide (WIDGET ("AppletVImage"));
      gtk_widget_hide (WIDGET ("AppletTitleLabel"));
      gtk_widget_hide (WIDGET ("AppletHUnreadLabel"));
      gtk_widget_show (WIDGET ("AppletHImage"));
      gtk_label_set_text (GTK_LABEL(WIDGET ("AppletVUnreadLabel")), locale_to_utf8 (unread+7));
    }
    else {
      gtk_widget_show (WIDGET ("AppletVUnreadLabel"));
      gtk_widget_show (WIDGET ("AppletVImage"));
      gtk_widget_hide (WIDGET ("AppletHUnreadLabel"));
      gtk_widget_hide (WIDGET ("AppletHImage"));
    }
    if (gdk_pixbuf_get_height (gdkpix) > size) {
      gtk_widget_hide (WIDGET ("AppletVImage"));
      gtk_widget_hide (WIDGET ("AppletHImage"));
    }
  }
  /* The panel is oriented vertically */
  else {
    gtk_widget_show (WIDGET ("AppletVUnreadLabel"));
    gtk_widget_show (WIDGET ("AppletVImage"));
    gtk_widget_hide (WIDGET ("AppletHUnreadLabel"));
    gtk_widget_hide (WIDGET ("AppletHImage"));
    if (size <= 36) {
      gtk_widget_hide (WIDGET ("AppletTitleLabel"));
      gtk_label_set_text (GTK_LABEL(WIDGET ("AppletVUnreadLabel")), locale_to_utf8 (unread+7));
    }
    if (gdk_pixbuf_get_width (gdkpix) > size){
      gtk_widget_hide (WIDGET ("AppletVImage"));
      gtk_widget_hide (WIDGET ("AppletHImage"));
    }
  }
};
#endif /* ifdef USE_GNOME */
